/* +=======================================================+ */
/* | First try with Three.js and WebGL                     | */
/* +-------------------------------------------------------+ */
/* | A CODEF submission for the WAB Contest #1             | */
/* | Released on june 30 2016                              | */
/* | Credits :                                             | */
/* |   - Code  : Dyno                                      | */
/* |   - Music : Dentrotheme by Domm / Animal Mine         | */
/* | More info on https://www.facebook.com/groups/codef/   | */
/* +-------------------------------------------------------+ */
/* | Copyleft 2016 by Dyno <dyno@aldabase.com>             | */
/* +=======================================================+ */

// ==ClosureCompiler==
// http://closure-compiler.appspot.com/home
// @compilation_level SIMPLE_OPTIMIZATIONS
// @output_file_name index.js
// @output_format text
// @output_info compiled_code
// ==/ClosureCompiler==

var first = 1,
    intro_pos = -1;

/*------------------*/
/* HELPER FUNCTIONS */
/*------------------*/

function buildScene() {
    // Create a Scene
    scene = new THREE.Scene();
    // Create a Camera in this WEBGL environment
    camera = new THREE.PerspectiveCamera(35, 1.33 /*800/600*/, 1, 2000);
    camera.position.z = 960;
    scene.add(camera);
    // Create an Ambiant Light in the scene
    scene.add(new THREE.AmbientLight(0x101010));
    // Create an empty group to place stuff in
    group = new THREE.Object3D();
    scene.add(group);
    // Create a directional light
    var directionalLight = new THREE.DirectionalLight(0xffffff, 1.0);
    directionalLight.position.set(1, 0, 1);
    scene.add(directionalLight);
}

/*------------*/
/* INTRO PART */
/*------------*/

function initIntro() {

    iteration = 0;
    buildScene();

    // Intro Object
    intro_pos++;
    text.drawPart(text_canvas, 0, 0, 0, (intro_pos % 3) * 36, 84, 36);
    var geometry = new THREE.SphereGeometry(10, 16, 16);
    imageData = text_canvas.contex.getImageData(0, 0, 84, 36);
    pix = imageData.data;
    for (var j = 0 ; j < 36 ; j++) {
        for (var i = 0 ; i < 84 ; i++) {
            var color = (pix[j*84*4+i*4+0] << 16)+(pix[j*84*4+i*4+1] << 8)+(pix[j*84*4+i*4+2]);
            if (color != 0) {
                var material = new THREE.MeshLambertMaterial({color: color, shading: THREE.SmoothShading});
                var sphere = new THREE.Mesh(geometry, material);
                sphere.position.x = i * 8 - 84 * 4;
                sphere.position.y = (18 - j) * 8;
                sphere.position.z = 16 * Math.random();
                group.add(sphere);
            }
        }
    }

    // Intro Shader
    glsl_alpha = 1.0;
    glsl = Glsl({
        canvas: glsl_canvas,
        fragment:
            "#ifdef GL_ES\n" +
            "precision mediump float;\n" +
            "#endif\n" +
             "uniform float t;\n" +
             "uniform vec2 resolution;\n" +
             "uniform float a;\n" +
             "float b(float f){" +
               "return fract(sin(f)*4358.55);" +
             "}" +
             "float b(vec2 f,vec2 l,float v){" +
               "return length(max(abs(f)-l,0.))-v;" +
             "}" +
             "void main(){" +
               "vec2 v=2.*gl_FragCoord.xy/resolution.xy-1.;" +
               "vec3 m=vec3(0.);" +
               "for(int i=0;i<32;i++)" +
                 "{" +
                   "float s=b(float(i)*1.987)*.01+.005,a=3.*b(float(i)*1.491)-1.5-.05*t*s,r=1.-2.*float(i)/32.,g=.4+b(float(i)*1.719)*.6,o=.15+b(float(i)*1.645)*.1,e=o*.66;" +
                   "if(a<-1.5)" +
                     "a+=3.*(floor(-(a+1.5)/3.)+1.);" +
                   "float c=b(v-vec2(a,r),vec2(o,e),.02),n=g*smoothstep(.22,0.,c)*.25,u=.2*g*smoothstep(.002,0.,c),h=.3*g*smoothstep(-.002,c,.007);" +
                   "m+=n*vec3(.0,.6,.9)+u*g+h;" +
                 "}" +
               "gl_FragColor=vec4(m*a,1.);" +
             "}",
        variables: {
            t: 0.,
            a: 1.
        },
        update: function (t) {
            this.set("t", t);
            this.set("a", glsl_alpha);
        }
    }).start();

    // Anim Intro
    animIntro();

}

function animIntro() {

    // 30 sec : jump to main part
    t2 = new Date();
    if (t2 - t1 > 30000) {
        t1 = t2;
        glsl.stop();
        initMain();
        return
    }    

    // Clear our 2D codef Canvas
    main_canvas.clear();
    // Intro timings
    if (first && (iteration < 240)) {
        glsl_alpha = 0;                         // Black screen
    } else if (first && (iteration < 300)) {
        glsl_alpha = (iteration - 240) / 60;    // Fade in
    } else if (first && (iteration < 540)) {
        glsl_alpha = 1;                         // Wait
    } else {
        glsl_alpha = 1;
        group.rotation.y -= 0.01;               // Rotate
    }
    // Render our hidden WebGL scene
    gl_canvas.render(scene, camera);
    // Draw the Webgl Scene to our 2D codef Canvas
    main_canvas.contex.globalAlpha = glsl_alpha;
    main_canvas.contex.drawImage(gl_canvas.domElement, 0, 0);
    // Next frame
    requestAnimFrame(animIntro);
    iteration++;
    updatestats();
}

/*-----------*/
/* MAIN PART */
/*-----------*/

/**
 * Hilbert Curve: Generates 2D-Coordinates in a very fast way.
 * @link https://github.com/mrdoob/three.js/blob/master/examples/js/geometries/hilbert3D.js
 */

function arrayAdd(tmp, hilbert) {
    return Array.prototype.push.apply(tmp, hilbert);
}

function hilbert3D(center, size, iterations, v0, v1, v2, v3, v4, v5, v6, v7) {
	var half = size / 2;
	var vec_s = [
		new THREE.Vector3(center.x - half, center.y + half, center.z - half),
		new THREE.Vector3(center.x - half, center.y + half, center.z + half),
		new THREE.Vector3(center.x - half, center.y - half, center.z + half),
		new THREE.Vector3(center.x - half, center.y - half, center.z - half),
		new THREE.Vector3(center.x + half, center.y - half, center.z - half),
		new THREE.Vector3(center.x + half, center.y - half, center.z + half),
		new THREE.Vector3(center.x + half, center.y + half, center.z + half),
		new THREE.Vector3(center.x + half, center.y + half, center.z - half)
	];
    var vec = [
        vec_s[v0],
        vec_s[v1],
        vec_s[v2],
        vec_s[v3],
		vec_s[v4],
        vec_s[v5],
        vec_s[v6],
        vec_s[v7]
	];
	if (--iterations >= 0) {
		var tmp = [];
        arrayAdd(tmp, hilbert3D(vec[0], half, iterations, v0, v3, v4, v7, v6, v5, v2, v1));
		arrayAdd(tmp, hilbert3D(vec[1], half, iterations, v0, v7, v6, v1, v2, v5, v4, v3));
		arrayAdd(tmp, hilbert3D(vec[2], half, iterations, v0, v7, v6, v1, v2, v5, v4, v3));
		arrayAdd(tmp, hilbert3D(vec[3], half, iterations, v2, v3, v0, v1, v6, v7, v4, v5));
		arrayAdd(tmp, hilbert3D(vec[4], half, iterations, v2, v3, v0, v1, v6, v7, v4, v5));
		arrayAdd(tmp, hilbert3D(vec[5], half, iterations, v4, v3, v2, v5, v6, v1, v0, v7));
        arrayAdd(tmp, hilbert3D(vec[6], half, iterations, v4, v3, v2, v5, v6, v1, v0, v7));
		arrayAdd(tmp, hilbert3D(vec[7], half, iterations, v6, v5, v2, v1, v0, v3, v4, v7));
		return tmp;
	}
	return vec;
}

function initMain() {
    iteration = 0;
    buildScene();
    
    // Main Object
    var geometry = new THREE.Geometry(),
        material,
        line,
        points = hilbert3D(new THREE.Vector3(0,0,0), 200.0, 2, 0, 1, 2, 3, 4, 5, 6, 7);
    geometry.colors = [];
    for (var i = 0 ; i < points.length - 1 ; i++) {
        geometry.vertices.push(points[i]);
        geometry.colors[i] = new THREE.Color( 0xffffff );
        if (intro_pos % 2) {
            geometry.colors[i].setHSL(0.4, 1.0, Math.max(0, (200 - points[i].x) / 400) * 0.5);
        } else {
            geometry.colors[i].setHSL(Math.max(0, (200 - points[i].x) / 400) * 1, 1.0, 0.4);
        }
    }
    material = new THREE.LineBasicMaterial({linewidth: 5, vertexColors: THREE.VertexColors});
    line = new THREE.Line(geometry, material);
    group.add(line);

    // Main Shader
    glsl = Glsl({
        canvas: glsl_canvas,
        fragment:
            "#ifdef GL_ES\n" +
            "precision mediump float;\n" +
            "#endif\n" +
            "uniform float t;\n" +
            "uniform vec2 resolution;\n" +
            "vec4 h(vec3 c){" +
                "vec4 k=vec4(1.,2./3.,1./3.,3.);" +
                "vec3 p=abs(fract(c.xxx+k.xyz)*6.-k.www);" +
                "return vec4(c.z*mix(k.xxx,clamp(p-k.xxx,0.,1.),c.y),1.);" +
            "}" +
            "void main(void){" +
                "vec2 v=2.*gl_FragCoord.xy/resolution.xy-1.;" +
                "vec2 d=vec2(sin(t),cos(t));" +
                "float c=" +
                    "sin(.02*dot(gl_FragCoord.xy,d))+"+
                    "cos(12.*length(v.xy-d))+" +
                    "t;" +
                "gl_FragColor=h(vec3(c/4.,.6,1.));" +
            "}",
        variables: {
            t: 0.,
        },
        update: function (t) {
            this.set("t", t / 1000);
        }
    }).start();

    // Anim main
    animMain();

}

function animMain() {

    // 30 sec : jump to intro part
    t2 = new Date();
    if (t2 - t1 > 30000) {
        t1 = t2;
        glsl.stop();
        initIntro();
        return
    }    

    // Clear our 2D codef Canvas
    main_canvas.clear();
    // Render our hidden WebGL scene
    gl_canvas.render(scene, camera);
    // Draw the Webgl Scene to our 2D codef Canvas
    main_canvas.contex.drawImage(gl_canvas.domElement, 0, 0);
    // Rotate our Group
    group.position.x = 120 * Math.sin(iteration / 23);
    group.position.z = 400 * Math.sin(iteration / 29) - 300;
    group.rotation.x += 0.03;
    group.rotation.y += 0.02;
    // Next frame
    requestAnimFrame(animMain);
    iteration++;
    updatestats();
    first = 0;
}

/*---------------*/
/* INIT FUNCTION */
/*---------------*/

function init() {
    text_canvas = new canvas(84, 36);
    text = new image('t.png');
    // Codef Canvas
    main_canvas = new canvas(800, 600, 'main');
    main_canvas.canvas.style.position = "absolute";
    main_canvas.canvas.style.left = "auto";
    main_canvas.canvas.style.top = "8px";
    // WebGL Canvas
    gl_canvas = new THREE.WebGLRenderer({ alpha: true });
    gl_canvas.setSize(800, 600);
    // GLSL Canvas
    glsl_canvas = document.createElement("canvas");
    glsl_canvas.setAttribute("width", 800);
    glsl_canvas.setAttribute("height", 600);
    document.getElementById("main").appendChild(glsl_canvas);
    // Music player
    new music().LoadAndRun('d.mod');
    // When text image is loaded
    text.img.onload = function () {
        // Init timer
        t1 = new Date();
        // Jump to intro scene
        initIntro();
    }
}

/*------------------*/
/* CLOSURE COMPILER */
/*------------------*/

window['init'] = init;
window['T'] = THREE;
